
;This function is used to despike the spectral domain in a CRISM
;using an ENVI slice
;Spikes are corrected through simple interpolation
;Inputs: 
;        thresh - the threshold value above which a channel in the
;                 residual spectrum is considered to be bad
;Returns a despiked version of the slice

;History: 
;May_08_2008: Mario Parente: vectorized version of previous IDl translation from Matlab.  Speed-up
;jun_23_2008: Mario & Trevor: reduced dimensionality from full cube to three_band to Save Memory
;

;INDEXING IS OFF BY DIFFERENCE BETWEEN SIZE OF THE TWO INDECES
function spectral_despike, data, thresh, SPIKE = spike_tracker
if(KEYWORD_SET(spike_tracker) eq 0) then spike_tracker=0
im = data.image
s=size(im)
ns=s[1]
nb=s[2]

im2=im

    diff1=im[*,1:nb-2]-im[*,0:nb-3]
    diff2=im[*,1:nb-2]-im[*,2:nb-1]
    ;the median of abs() represents about .67 * robust std. dev
    spec_thresh = thresh*median(abs(diff1))

    mag1=diff1 GE 0 ;1 if positive, 0 if negative
    mag2=diff2 GE 0 ;1 if positive, 0 if negative
    
    i=where((mag1 eq mag2 ) AND (abs(diff1) GT spec_thresh) AND (abs(diff2) GT spec_thresh))
    
    ;note that this correction will not fully correct when spikes are next to eachother
    Avg=(im[*,0:nb-3] +im[*,2:nb-1])/2.
    
    if (i[0] ne -1) then begin
      im[i+ns]=Avg[i]
      if(spike_tracker eq 1) then data.spike_mask[i+ns]=data.spike_mask[i+ns]+.5
    endif

data.image = im
; and we are done.  Return!
return, data
end
