; PURPOSE: Empirical normalizaion of detector-oriented striping.

;10/10/2006 (fps)
;   Original incarnation. Empirical flat-field based on scene content. Rescale column stripes.
;10/11/2006 (fps)
;   Added ignore_values and mask_value options. Expliticty filter out non-finite values.
;10/12/2006 (fps)
;   Added show_image_planes and show_pixel_mask options.
;10/13/2006 (fps)
;   Added column_kernel option. Scale column to local column kernel of specified size.
;11/30/2006 (fps)
;   Added row stripe romoval via image transposition; column_kernel size becomes row kernel size
;   Fixed failure mode when all pixels in band are inactive
;12/01/2006 (fps)
;   Added median stats option
;   Added sigma stats option
;12/06/2006 (fps)
;   Added percentile stats option
;05/06/2008 (fps)
;   Added additive operator option
;05/07/2008 (fps)
;   Comment out superfluous print and help statements
;10/28/2009 (mfm)
;   Comment out more print statements

function mro_crism_remove_stripes, in_image, ignore_values = ignore_values, mask_value = mask_value, column_kernel = column_kernel, $
    	    	    	    	   show_image_planes = show_image_planes, show_pixel_mask = show_pixel_mask, $
				   transpose_image = transpose_image, median_stats = median_stats, additive_operator = additive_operator, $
				   sigma_stats = sigma_stats, percentile_stats = percentile_stats

;print, 'MRO_CRISM_REMOVE_STRIPES'

if (keyword_set(transpose_image)) then begin
    transpose_axes = lindgen((size(in_image))[0])
    transpose_axes[0] = 1
    transpose_axes[1] = 0

    image = transpose(in_image, transpose_axes)
endif else begin
    image = in_image
endelse

image_size = size(image)
;print, image_size

if not(keyword_set(ignore_values)) then begin
    ignore_values = ['65535']
endif

num_ignore_values = n_elements(ignore_values)


if not(keyword_set(mask_value)) then begin
    mask_value = 65535
endif

show_image_planes = keyword_set(show_image_planes)
show_pixel_mask = keyword_set(show_pixel_mask)


image_out = make_array(size=image_size, value = mask_value, /float)
;help, image_out

if (image_size[0] EQ 2) then begin
    band_max = 1
endif else begin
    band_max = image_size[image_size[0]]
endelse

column_max = image_size[1]
;print, 'Column Max: ', column_max

if (keyword_set(column_kernel)) then begin

    if ((column_kernel MOD 2) NE 1) then begin
    	column_kernel = column_kernel + 1
    	print, 'MRO_CRISM_REMOVE_STRIPES: kernel size is not odd - adding one
	print, 'MRO_CRISM_REMOVE_STRIPES: kernel size = ' + strtrim(string(column_kernel),2)
    endif

endif


for k = 0, band_max -1 do begin

    image_plane = image[*,*,k]
    active_pixel_indx = make_array(n_elements(image_plane), value = 1)
    
    ;Specified ignore values
    
    for i = 0, num_ignore_values -1 do begin
    ;	print, i
    	ignore_indx = where(image_plane EQ ignore_values[i], num_ignore_indx)
;	print, 'Num Ignore Indx: ', num_ignore_indx
	if (num_ignore_indx GT 0) then begin
;	    print, 'Flagging Pixels...'
	    active_pixel_indx[ignore_indx] = 0
	endif
    endfor    
  
    ;Non-finite values
    non_finite_indx = where(finite(image_plane) EQ 0, num_non_finite_indx)

    if (num_non_finite_indx GT 0) then begin
      	active_pixel_indx[non_finite_indx] = 0
    endif

    num_active_pixel_indx = total(active_pixel_indx)
    ;print, num_active_pixel_indx


    
    ;sigma stats
    if (keyword_set(sigma_stats)) then begin
    	stats_indx = where(active_pixel_indx EQ 1, num_stats_indx)
    	image_plane_sigma = stddev(image_plane[stats_indx])
	image_plane_mean = mean(image_plane[stats_indx])
	sigma_cil = image_plane_mean + sigma_stats * image_plane_sigma
	sigma_flr = image_plane_mean - sigma_stats * image_plane_sigma
	
	sigma_threshold_indx = where((image_plane[stats_indx] GE sigma_cil) OR (image_plane[stats_indx] LE sigma_flr), num_sigma_threshold)

;    	print, image_plane_sigma
;	print, image_plane_mean
;    	print, sigma_cil
;	print, sigma_flr
;	print, num_sigma_threshold
;	print, num_active_pixel_indx
;	print, float(num_sigma_threshold) / float(num_active_pixel_indx)

    	if (num_sigma_threshold GT 0) then begin
	    active_pixel_indx[stats_indx[sigma_threshold_indx]] = 2
	endif

    
    endif

    ;percentile stats
    if (keyword_set(percentile_stats)) then begin
    	stats_indx = where(active_pixel_indx EQ 1)
    	image_plane_percentiles = percentiles(image_plane[stats_indx])
	
	pct_cil_indx = 100 - ((100 - percentile_stats) / 2)
	pct_flr_indx = ((100 - percentile_stats) / 2)
	
	pct_cil = image_plane_percentiles[pct_cil_indx]
	pct_flr = image_plane_percentiles[pct_flr_indx]
	
	percentile_threshold_indx = where((image_plane[stats_indx] GE pct_cil) OR (image_plane[stats_indx] LE pct_flr), num_percentile_threshold)

;    	print, image_plane_percentiles[50]   ;median
;	print, pct_cil
;	print, pct_flr
;	print, num_percentile_threshold
;	print, num_active_pixel_indx
;	print, float(num_percentile_threshold) / float(num_active_pixel_indx)

    	if (num_percentile_threshold GT 0) then begin
	    active_pixel_indx[stats_indx[percentile_threshold_indx]] = 2
	endif

    
    endif


    if (num_active_pixel_indx GT 0) then begin
  
        if (keyword_set(median_stats)) then begin
            ;median
            image_plane_value = median(image_plane[where(active_pixel_indx EQ 1, num_image_plane_indx)])
    	endif else begin
    	    ;mean
    	    image_plane_value = mean(image_plane[where(active_pixel_indx EQ 1, num_image_plane_indx)])
        endelse
        
    endif else begin
    	image_plane_value = 0.0
	num_image_plane_indx = long(0)
    endelse
    
    ;print, num_image_plane_indx
        
    active_pixel_indx = reform(active_pixel_indx, image_size[1], image_size[2])

    if (show_pixel_mask) then begin
    	if (k EQ 0) then begin
    	    window, 0, xsize = image_size[1], ysize = image_size[2], retain=2, title = 'Pixel Mask', xpos=0, ypos=0
    	endif
	wset, 0
    	tvscl, rotate(active_pixel_indx,7)
    endif
    

    for j = 0, column_max -1 do begin

    	image_plane_column = image_plane[j,*]
    	active_pixel_column_indx = where(active_pixel_indx[j,*] EQ 1, num_active_pixel_column_indx)

    	;print, num_active_pixel_column_indx

	if (num_active_pixel_column_indx GT 0) then begin

    	    ;print, 'col#: ', num_active_pixel_column_indx

    	    if (keyword_set(median_stats)) then begin
    	    	column_value = median(image_plane_column[active_pixel_column_indx])
	    endif else begin
	        column_value = mean(image_plane_column[active_pixel_column_indx])
	    endelse

    	    if (keyword_set(column_kernel)) then begin
    	    	col_indx = lindgen(column_kernel) + j - (column_kernel -1) / 2
	    
    	    	if (min(col_indx) LT 0) then col_indx = col_indx - min(col_indx)
    	    	if (max(col_indx) GT (column_max -1)) then col_indx = col_indx - (max(col_indx) - (column_max -1))

    	    	active_pixel_column_kernel_indx = where(active_pixel_indx[col_indx,*] EQ 1, num_active_column_pixel_column_indx)
    	
	    	if ( total( ( (image_plane[col_indx,*])[active_pixel_column_kernel_indx]) EQ 65535 ) NE 0 ) then stop
		
    	    	if (keyword_set(median_stats)) then begin
		    column_kernel_value = median( (image_plane[col_indx,*])[active_pixel_column_kernel_indx] )
		endif else begin
    	    	    column_kernel_value = mean( (image_plane[col_indx,*])[active_pixel_column_kernel_indx] )    	    
		endelse

    	    endif

    	    active_calc_pixel_column_indx = where(active_pixel_indx[j,*] NE 0, num_active_calc_pixel_column_indx)
	    
    	    if (keyword_set(column_kernel)) then begin
    	    	if (keyword_set(additive_operator)) then begin
		    destriped_data = image_plane_column[active_calc_pixel_column_indx]  - (column_value - column_kernel_value)
		endif else begin
		    destriped_data = image_plane_column[active_calc_pixel_column_indx] * column_kernel_value / column_value
		endelse
	    endif else begin
    	    	if (keyword_set(additive_operator)) then begin
		    destriped_data = image_plane_column[active_calc_pixel_column_indx]  - (column_value - image_plane_value)
		endif else begin
    	    	    destriped_data = image_plane_column[active_calc_pixel_column_indx] * image_plane_value / column_value
		endelse
    	    endelse
	    
    	    image_out[j, active_calc_pixel_column_indx, k] = destriped_data
	    
    	endif
	
    
    endfor

    if (show_image_planes) then begin
    	min_val = min(image_plane[where(active_pixel_indx EQ 1)])
	max_val = max(image_plane[where(active_pixel_indx EQ 1)])

        if (k EQ 0) then begin
    	    window, 1, xsize = image_size[1], ysize = image_size[2], retain=2, title = 'Input Image Plane', xpos = 0, ypos = image_size[2] + 120
    	    window, 2, xsize = image_size[1], ysize = image_size[2], retain=2, title = 'Output Image Plane', xpos = image_size[1] + 40, ypos = image_size[2] + 120
        endif

    	wset, 1
	tv, rotate(bytscl(image_plane, min = min_val, max = max_val),7)
	wset, 2
	tv, rotate(bytscl(image_out[*,*,k], min = min_val, max = max_val),7)

	
    endif



endfor

if (keyword_set(transpose_image)) then begin
    image_out = transpose(image_out, transpose_axes)
endif

return, image_out

END
