;03/10/2009 (fps)
;   Minimal code cleanup for CAT 6.5 release
;10/15/2015 (FM)
;	Modified to ignor CRISM NAN 65535 pixels in stats whether input mask includes them
;   or not.

function mro_crism_spectral_stats, image, mask=mask, median_select=median_select, mean_select=mean_select, $
				   stddev_plus_select=stddev_plus_select, stddev_minus_select=stddev_minus_select, $
    	    	    	    	   min_select=min_select, max_select=max_select

mean_select = keyword_set(mean_select)
median_select = keyword_set(median_select)
min_select = keyword_set(min_select)
max_select = keyword_set(max_select)
stddev_plus_select = keyword_set(stddev_plus_select)
stddev_minus_select = keyword_set(stddev_minus_select)


if (total([median_select, mean_select, min_select, max_select, stddev_plus_select, stddev_minus_select]) EQ 0) then begin
    mean_select = 1
    median_select = 0
    min_select = 0
    max_select = 0
    stddev_plus_select = 1
    stddev_minus_select = 1
endif

if (((stddev_plus_select EQ 1) OR (stddev_minus_select EQ 1)) AND (mean_select EQ 0)) then begin
    mean_select = 1
endif

;['Maximum', 'Mean + Stddev', 'Mean', 'Median', 'Mean - Stddev', 'Minimum']

active_indx_vector = [max_select, stddev_plus_select, mean_select, median_select, stddev_minus_select, min_select]
active_indx = where(active_indx_vector EQ 1, num_active_indx)

;print, active_indx

image_size = size(image)

mask_val = 65535.0

if (image_size[0] EQ 2) then begin
    nb = 1
endif else begin
    nb = image_size[image_size[0]]
endelse

; Added 10/15/2015, FM:
; Make a working mask array adding CRISM NAN 65535 pixels:
if (keyword_set(mask)) then begin
	wkmask = mask
endif else begin
	wkmask = fix(image)*0 + 1
endelse
nanpix = where((image eq 65535.0), nnan)
if (nnan gt 0) then wkmask[nanpix] = 0
; Using "wkmask" in place of input mask excludes all 65535 pixels also:

if (keyword_set(wkmask)) then begin
    active_pixels = where(wkmask EQ 1, num_active_pixels)
endif else begin
    active_pixels = lindgen(image_size[1] * image_size[2])
    num_active_pixels = n_elements(active_pixels)
endelse

;print, num_active_pixels

if (num_active_pixels GT 0) then begin

    output_spectra = make_array(nb, num_active_indx, value = mask_val, /float)

    for k = 0, nb -1 do begin
    	image_plane = image[*,*,k]

    	image_plane_min = mask_val
    	image_plane_minus_stddev = mask_val
	image_plane_mean = mask_val
	image_plane_median = mask_val
	image_plane_plus_stddev = mask_val
	image_plane_max = mask_val


    	if (total(active_indx EQ 5) NE 0) then begin
	    ;MIN
	    ;print, 'MIN'
	    image_plane_min = min(image_plane[active_pixels])
	endif

	if (total(active_indx EQ 0) NE 0) then begin
	    ;MAX
	    ;print, 'MAX'
	    image_plane_max = max(image_plane[active_pixels])
	endif

	if (total(active_indx EQ 3) NE 0) then begin
	    ;MEDIAN
	    ;print, 'MEDIAN'
	    image_plane_median = median(image_plane[active_pixels])
	endif

	if (total(active_indx EQ 2) NE 0) then begin
	    ;MEAN
	    ;print, 'MEAN'
	    image_plane_mean = mean(image_plane[active_pixels])
	endif

	if ((total(active_indx EQ 1) NE 0) OR (total(active_indx EQ 4) NE 0)) then begin
	    ;MINUS/PLUS STDDEV (always paired)
	    ;print, 'MINUS/PLUS STDDEV'
	    image_plane_stddev = stddev(image_plane[active_pixels])

	    image_plane_plus_stddev = image_plane_mean + image_plane_stddev
	    image_plane_minus_stddev = image_plane_mean - image_plane_stddev
	endif

	output_vector = [image_plane_max, image_plane_plus_stddev, image_plane_mean, image_plane_median, image_plane_minus_stddev, image_plane_min]
	;print, output_vector

	output_spectra[k, *] = output_vector[active_indx]
	;print, output_spectra[k, active_indx]

    endfor

endif else begin
    output_spectra = 0.0
endelse


return, output_spectra
END
