;******************************************************************************
;
; function algk_temperature_event
;
; CAT/ENVI event handler to generate surface temperature image using AlgK 
; thermal model. 
;
; INPUT:
;
; OUTPUT:
;	Parameters:    None
;	Return value:  2D array of surface temperatures.
;
; KEYWORDS: none
;
; EXAMPLE:
;
; NOTES:
;		
; USES:
;	No external procedures.
;
; HISTORY:
;	2008 Mar ??   F. Morgan: First version written.
;
;******************************************************************************

pro algk_temperature_event, ev


widget_control, ev.id, get_uvalue=uvalue

; Simple catch to prevent crash:
catch, error
if (error NE 0) then begin
	cat_error_report, routine_name='algk_temperature_event'
	return
endif

; Select an open image file:
envi_select, fid=in_fid, pos=in_bands, dims=sf_dims, $
	/file_only, /no_dims, title='Select an open image file:'
if (in_fid[0] eq -1) then return                  ;cancel selected

; Get existing header data:
envi_file_query, in_fid, fname=fnm, ns=ns,nl=nl,nb=nb, $
		interleave=interleave, data_type=dt, $
		byte_swap=bo, wavelength_units=wu, wl=waves, fwhm=bwlbl,$
		data_ignore_value=div, def_bands=def_bands, $
		descrip=descrip
; also get map_info:
map_info = envi_get_map_info(fid=in_fid)

; Get AlgK temperature image
stat = get_algk_temperature(fnm, temp)

; Load into available bands list
envi_enter_data, temp, data_ignore_value=65535., bnames=['AlgK Temperature']


return
end
