;10/15/2006 (fps)
;   Initial developemnt - envi widget interface to mro_crism_remove_stripes()
;11/02/2006 (smp)
;   Added traceability to input file by adding descriptor to ENVI hdr.
;11/30/2006 (fps)
;   Revised widget interface to allow row or column stripe removal
;   Replaced all code reference to 'column kernel' with 'local kernel'
;05/06/2008 (fps)
;   Added input statistics thresholding options to widget interface
;   Added normaliztion statistic and operator options to widget interface
;   Parse auto-managed widget return values and handle all mro_crism_remove_stripes() keywords
;05/07/2008 (fps)
;   Fixed bug in parsing of widget return struct
;   Support bnames as well as wavelengths in output ENVI header
;   Comment out superfluous print and help statements
;03/10/2009 (fps)
;   Code cleanup for CAT 6.5 release
;   Added release/development differentiation
;   	Render advanced options in the interface widget only if configuration file keyword XXXXX = 1
;   	Used some chicanery to make reduced interface widget look OK
;   Disabled band subset selection and spatial subset selection
;   Changed default output filename history code from DEST to RST
;03/12/2009 (mfm)
;	Use config switch to turn development version menu items on and off.
;10/28/2009 (mfm)
;   Comment out more superfluous print and help statements
;12/05/2009 (mfm)
;   Use get_cat_header_data and write_cat_header for header IO
;11/17/2010 (mfm)
;   add header parameter CAT_PRODUCT_VERSION
;11/18/2010 (mfm)
;   add four more header parameters



pro mro_crism_remove_stripes_envi, event

; Check config param to switch menu functionality
devver = strtrim(get_catcfg_param('CAT_DEV_VERSION'),2)
if (devver ne '1') then map_optional=0 else map_optional=1
 

;Set compiler options
compile_opt strictarr

;Simple catch to prevent crash
catch, error
if (error NE 0) then begin
  cat_error_report, routine_name='mro_crism_remove_stripes_envi'
  return
endif

;Select input band from available bands list
envi_select, fid=in_fid, pos=in_pos, dims=in_dims, file_type=in_file_type, title='MRO CRISM REMOVE STRIPES: Select Input File', /file_only, /no_spec, /no_dims

if (in_fid[0] eq -1) then begin
    print, 'MRO CRISM REMOVE STRIPES: Cancel/Invalid FID'
    return
endif

;Gather some useful info
stat = get_cat_header_data(in_fid, NS=in_ns, NL=in_nl, NB=in_nb, WL=in_wl, $
	WAVELENGTH_UNITS=wu, BBL=bbl, FWHM=fwhm, BNAMES=in_bnames, FNAME=in_filename, $
	INTERLEAVE=interleave, OFFSET=offset, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DESCRIP=descrip, MAP_INFO=in_map_info, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files )

if (stat ne 0) then begin
	ok = dialog_message( 'Problem getting header data from '+file_basename(in_filename) $
	                     + string(10b) + 'Aborting procedure.', $
		                     title='Remove Stripes Error')
	return
endif

;Construct default output filenames
last_slash = strpos(in_filename, '/', /reverse_search) > strpos(in_filename, '\', /reverse_search)
last_dot = strpos(in_filename, '.', /reverse_search)

in_file_directory = strmid(in_filename, 0, last_slash + 1)
in_file_basename = strmid(in_filename, last_slash + 1, last_dot - last_slash -1)
if (last_dot NE -1) then begin
	in_file_extension = strmid(in_filename, last_dot)
endif else begin
	in_file_extension = ''
endelse

output_file = in_file_directory + in_file_basename + '_RST' + in_file_extension


;Gather user input info via ENVI auto managed widgets
auto_base = widget_auto_base(title = 'MRO CRISM REMOVE STRIPES: Caveat Emptor!')
main_base = widget_base(auto_base, column = 2)

    processing_base = widget_base(main_base, /frame, column = map_optional)
    	default_processing_base = widget_base(processing_base, /column)

    	procesing_label = widget_label(default_processing_base, value = 'Processing Options')
    	
	column_row_base = widget_base(default_processing_base, /column, /frame)
	    column_row_select_list = ['Column Stripes', 'Row Stripes']
	    column_row_prompt = 'Orientation:'
    	    column_row_select = widget_menu(column_row_base, /exclusive, list = column_row_select_list, uvalue = 'column_row_select', prompt = column_row_prompt, /auto_manage, default_ptr = 0, /row)

    	mean_median_base = widget_base(default_processing_base, /column, /frame)
    	    mean_median_select_list = ['Mean', 'Median']
	    mean_median_prompt = 'Normalization Statistic:'
    	    mean_median_select = widget_menu(mean_median_base, /exclusive, list = mean_median_select_list, uvalue = 'mean_median_select', prompt = mean_median_prompt, /auto_manage, default_ptr = 0)	

    	mult_add_base = widget_base(default_processing_base, /column, /frame)
	    mult_add_select_list = ['Multiplicative', 'Additive']
	    mult_add_prompt = 'Normalization Operator:'
    	    mult_add_select = widget_menu(mult_add_base, /exclusive, list = mult_add_select_list, uvalue = 'mult_add_select', prompt = mult_add_prompt, /auto_manage, default_ptr = 0)	


    	advanced_processing_base = widget_base(processing_base, /column, map = map_optional)		    	
	local_kernel_base = widget_base(advanced_processing_base, /column, /frame)
    	    local_kernel_select_list = ['Full Scene', 'Local Kernel']
	    local_kernel_prompt = 'Kernel Scope:'
	    local_kernel_select = widget_menu(local_kernel_base, /exclusive, list = local_kernel_select_list, uvalue = 'local_kernel_select', prompt = local_kernel_prompt, /auto_manage, default_ptr = 0)
    	
	    local_kernel_param_base = widget_base(local_kernel_base, /row)
    	    	local_kernel_value_default = 25
    	    	local_kernel_param = widget_param(local_kernel_param_base, default = local_kernel_value_default, prompt = 'Local Kernel Size:', $
    	    	    	    	    	      	  uvalue = 'local_kernel_param', /auto_manage, dt = 2)

    	input_threshold_base = widget_base(advanced_processing_base, /column, /frame)
    	    threshold_prompt = 'Input Thresholding:'
    	    threshold_select_list = ['None', 'Sigma', 'Percentile']
	    threshold_select = widget_menu(input_threshold_base, /exclusive, list = threshold_select_list, uvalue = 'threshold_select', prompt = threshold_prompt, /auto_manage, default_ptr = 0)

    	    threshold_param_base = widget_base(input_threshold_base, /row)
    	    	threshold_value_default = 3
    	    	threshold_param = widget_param(threshold_param_base, default = threshold_value_default, prompt = 'Threshold Parameter:', $
    	    	    	    	    	       uvalue = 'threshold_param', /auto_manage, dt = 2)

    format_base = widget_base(main_base, /frame, column = map_optional)
    advanced_format_base = widget_base(format_base, /column, map = map_optional)
    
    ignore_mask_base = widget_base(advanced_format_base, /column, /frame)

    	ignore_values_base = widget_base(ignore_mask_base, /column)
    	    ignore_values_default = '[65535.0]'
    	    ignore_values_prompt = 'Finite Data Ignore Values (IDL array syntax):'
    	    ignore_values_input = widget_string(ignore_values_base, prompt = ignore_values_prompt, default = ignore_values_default, $
    	    	    	    	    	    	/auto_manage, uvalue = 'ignore_values_input')

    	mask_pixel_value_base = widget_base(ignore_mask_base, /row)
    	    mask_pixel_value_default = 65535.0
    	    mask_pixel_value_prompt = 'Ignore Pixel Return Value:'
    	    mask_pixel_value_param = widget_param(mask_pixel_value_base, prompt = mask_pixel_value_prompt, default = mask_pixel_value_default, $
    	    	    	    	    	    	  /auto_manage, uvalue = 'mask_pixel_value_param')

    show_pixel_planes_base = widget_base(advanced_format_base, /column, /frame)

    	show_pixel_toggle_base = widget_base(show_pixel_planes_base, /row)
    	    show_pixel_toggle_list = ['No', 'Yes']
    	    show_pixel_toggle_prompt = 'Show Active Pixel Map?'
    	    show_pixel_toggle = widget_toggle(show_pixel_toggle_base, prompt = show_pixel_toggle_prompt, list = show_pixel_toggle_list, $
    	    	    	    	    	      uvalue = 'show_pixel_toggle', /auto_manage, default = 0)

    	show_planes_toggle_base = widget_base(show_pixel_planes_base, /row)
    	    show_planes_toggle_list = ['No', 'Yes']
    	    show_planes_toggle_prompt = 'Show Input/Output Image Planes?'
    	    show_planes_toggle = widget_toggle(show_planes_toggle_base, prompt = show_planes_toggle_prompt, list = show_planes_toggle_list, $
    	    	    	    	    	       uvalue = 'show_planes_toggle', /auto_manage, default = 0)

    remove_stripes_outfm = widget_outfm(format_base, /auto_manage, /frame, default = output_file, uvalue = 'remove_stripes_outfm')


result = auto_wid_mng(auto_base)

if (result.accept EQ 0) then begin
    print, 'MRO CRISM REMOVE STRIPES: Cancel
    return
endif

;help, result, /struct
;print, result

;stop

;Set output parameters
out_dims = in_dims
out_dt = 4 ;floating point
out_pos = in_pos
out_nb = n_elements(out_pos)

;memory check
mem_check = magic_mem_check(dims=in_dims, in_memory = result.remove_stripes_outfm.in_memory, $
			    out_dt=out_dt, nb=out_nb, out_name = result.remove_stripes_outfm.name)

if (mem_check.cancel EQ 1) then begin
    print, 'MRO CRISM REMOVE STRIPES: Cancel'
    return
endif

;Store result of memory check in result structure
result.remove_stripes_outfm.in_memory = mem_check.in_memory
result.remove_stripes_outfm.name = mem_check.out_name

in_samples = in_dims[2] - in_dims[1] + 1
in_lines = in_dims[4] - in_dims[3] + 1

;Destriping requies sequental processing of spatially complete spectral bands
;"Pseudo-Tiling"

;Decalare output band array
output_plane = make_array(in_samples, in_lines, value = 0.0, /float)

;Declare in-memory output array or open output file
if (result.remove_stripes_outfm.in_memory EQ 1) then begin
    output_cube = make_array(in_samples, in_lines, out_nb, value = 0.0, /float)
    out_status_string = 'Output to Memory'
endif else begin
    openw, out_lun, result.remove_stripes_outfm.name, /get_lun
    out_status_string = 'Output File: ' + result.remove_stripes_outfm.name
endelse


;Wrangle widget return info
show_pixel_mask = result.show_pixel_toggle
show_image_planes = result.show_planes_toggle
mask_value = result.mask_pixel_value_param

ignore_values_string = result.ignore_values_input
ignore_values_split = strsplit(ignore_values_string, ',[]', /extract)
ignore_values = float(ignore_values_split)

local_kernel = result.local_kernel_select * result.local_kernel_param
transpose_image = result.column_row_select

median_stats = result.mean_median_select
additive_operator = result.mult_add_select

sigma_stats = 0
percentile_stats = 0

if (result.threshold_select EQ 1) then begin
    sigma_stats = result.threshold_param
endif
if (result.threshold_select EQ 2) then begin
    percentile_stats = result.threshold_param
endif


;Assemble string to be displayed in ENVI status window
status_string_array = ['Input File: ' + in_filename, out_status_string]

envi_report_init, status_string_array, base = status_base, title = 'MRO CRISM REMOVE STRIPES', /interupt

envi_report_inc, status_base, out_nb


;Initialize pseudo tiling
;Main processing loop - one iteration for each band
for k = 0, out_nb -1 do begin

    envi_report_stat, status_base, k, out_nb, cancel=cancel

    if (cancel) then begin
    	print, 'MRO CRISM REMOVE STRIPES: Cancel'
    	envi_report_init, base = status_base, /finish

    	if (result.remove_stripes_outfm.in_memory EQ 0) then begin
    	    free_lun, out_lun
    	endif

    	return
    endif

    ;Grab spectral band
    in_data = envi_get_data(fid = in_fid, pos = out_pos[k], dims=in_dims)

    output_plane = mro_crism_remove_stripes(in_data, mask_value = mask_value, ignore_values = ignore_values, $
    	    	    	    	    	    column_kernel = local_kernel, transpose_image = transpose_image, $
					    show_pixel_mask = show_pixel_mask, show_image_planes = show_image_planes, $
					    median_stats = median_stats, additive_operator = additive_operator, $
					    sigma_stats = sigma_stats, percentile_stats = percentile_stats)

    ;Write band result to output file or store in output memory array
    if (result.remove_stripes_outfm.in_memory EQ 1) then begin
    	output_cube[*,*,k] = output_plane
    endif else begin
	writeu, out_lun, output_plane
    endelse

endfor

envi_report_init, base = status_base, /finish



;============================
;    HEADER PREP
;============================

if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Append history:
cat_history = cat_append_history(cat_history, 'RST')

; Update input file list:
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(in_filename)]
endif else begin
	cat_input_files = [file_basename(in_filename)]
endelse

; Spectral subsetting...
if (in_wl[0] ne -1) then begin
	wl_out = in_wl[out_pos]
endif else begin
	wl_out = -1
endelse
if (bbl[0] ne -1) then begin
	bbl_out = bbl[out_pos]
endif else begin
	bbl_out = -1
endelse
if (fwhm[0] ne -1) then begin
	fwhm_out = fwhm[out_pos]
endif else begin
	fwhm_out = -1
endelse
if (n_elements(in_bnames) gt 1) then begin
	bnames_out = in_bnames[out_pos]
endif else begin
	bnames_out = -1
endelse
if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	def_bands = crism_def_bands(cat_crism_detector_id, wl_out)
endif


;Register memory item in ENVI session or generate header for output file
if (result.remove_stripes_outfm.in_memory EQ 1) then begin
    envi_enter_data, output_cube, wl = in_wl[out_pos], bnames = in_bnames[out_pos], data_ignore_value = result.mask_pixel_value_param
endif else begin
    free_lun, out_lun

	stat = write_cat_header(FNAME=result.remove_stripes_outfm.name, $
	    NS=in_samples, NL=in_lines, NB=out_nb, $
	    DATA_TYPE=4, INTERLEAVE=0, OFFSET=0, $
	    BBL=bbl_out, BNAMES=in_bnames[out_pos], $
	    DATA_IGNORE_VALUE=result.mask_pixel_value_param, $
	    DEF_BANDS=def_bands, DESCRIP='CAT CRISM DATA stripes removed', $
	    FWHM=fwhm_out, MAP_INFO=in_map_info, $
	    WAVELENGTH_UNIT=wu, WL=wl_out, $
	    CAT_START_TIME          = cat_start_time, $
	    CAT_SCLK_START          = cat_sclk_start, $
	    CAT_CRISM_OBSID         = cat_crism_obsid, $
	    CAT_OBS_TYPE            = cat_obs_type, $
	    CAT_PRODUCT_VERSION     = cat_product_version, $
	    CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $
	    CAT_BIN_MODE            = cat_bin_mode, $
	    CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	    CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	    CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	    CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	    CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	    CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	    CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	    CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	    CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	    CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	    CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	    CAT_HISTORY             = cat_history, $
	    CAT_INPUT_FILES         = cat_input_files )

	; Report problems with header if any indicated by status returned:
	outfnm = result.remove_stripes_outfm.name
	if ((stat ne 0) and (~keyword_set(batch))) then begin
		if (stat eq -1) then begin
			msg = 'Problem writing header for '+file_basename(outfnm) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
			ok = dialog_message(msg, title='Header Error')
		endif else if (stat eq -2) then begin
			msg = 'Problem writing header for '+file_basename(outfnm) + string(10b) $
			      + 'Header already exists.'
			ok = dialog_message(msg, title='Header Error')
		endif else begin
			msg = 'Problem writing header for '+file_basename(outfnm) + string(10b) $
			      + 'Unspecified error.'
			ok = dialog_message(msg, title='Header Error')
		endelse
	endif

endelse

END

