;***************************************************************************************
;
; function cat_all_digits, input_string, allow_hex=allow_hex
;
; Determine whether an input string is made up entirely of numerical digits or not.
; (Returns false if input is empty string.)
;
; INPUT PARAMETERS:
;	input_string	string		Scalar string
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:
;	"true" (1) if all digits; "false" (0) if empty string or any character 
;	is not a digit.
;
; KEYWORDS
;	allow_hex    If set, alos accept a..f, A..F as digits
;
; EXAMPLES:
;	x = cat_all_digits('1483')
;		result: x = 1
;
;	x = cat_all_digits('')
;		result: x = 0
;
;	x = cat_all_digits('367 123')
;		result: x = 0
;
;	x = cat_all_digits('15f3')
;		result: x = 0
;
;	x = cat_all_digits('15f3',/allow_hex)
;		result: x = 1
;
; HISTORY:
;	2009 Sep 02   FM: First version
;
;***************************************************************************************

function cat_all_digits, input_string, allow_hex=allow_hex

if (keyword_set(allow_hex)) then begin
	return, stregex(input_string, '^[0-9A-Fa-f]+$', /boolean)
endif else begin
	return, stregex(input_string, '^[0-9]+$', /boolean)
endelse
end
