;******************************************************************************
;
; function cat_envi_find_display, fid
;
; Find the display number of the display group associated with open ENVI 
; file at file ID 'fid.'
;
; INPUT:
;	fid		   Open ENVI file ID.
;
; OUTPUT PARAMETERS:  none
;
; RETURN VALUE:
;	display number (zero offset), -1 if no display open for specified file.
;   (intarr if more than one display group is open for the same file)
;
; KEYWORDS: None
;
; NOTES:
;	Assumes all components of each display come from the same FID, 
;	which is not strictly necessary. (envi_disp_query returns a 3-element
;	vector for fid, corresponding for RGB images to the FID for each 
;	component. See ENVI help. Using only fid[0] here.)
;
; HISTORY:
;	2009 Dec 07   F. Morgan: First version.
;
;******************************************************************************

function cat_envi_find_display, fid

; Find all the display group numbers:
alldn = envi_get_display_numbers()
if (alldn[0] eq -1) then return,-1

; Check FID inpt:
if ((n_elements(fid) eq 0) || (fid[0] lt 0)) then return,-1

; Find the display...
nd = n_elements(alldn)
out_dn = -1
for i=0,nd-1 do begin
	envi_disp_query, alldn[i], fid=d_fid
	if (d_fid[0] eq fid[0]) then out_dn = [out_dn, alldn[i]]
endfor
k = where((out_dn ge 0), nk)
if (nk eq 0) then return,-1
out_dn = out_dn[1:*]

if (n_elements(out_dn) eq 1) then return,out_dn[0] else return,out_dn
end
