;******************************************************************************
;
; function cat_strip_nondigit, hex_string
;
; Strip non-numeric characters from a string that's intended to be a decimal 
; or hexadecimal value.
;
; INPUT:
;	input_string        string       Numeric value posibly with non-digit characters.
;
; OUTPUT PARAMETERS: none
;
; RETURN VALUE:
;	First sequence of valid hex digits; '' if none.
;
; KEYWORDS:
;	allow_hex      If set, keep hex digits A-F, a-f
;	allow_minus    If set, keep '-' character
;	allow_float    If set, keep '.' or '-'
;
; NOTES:
;	Strips the first non-digit characters, if any, leaves the first contiguous 
;	set of digits, strips everything following and including the next 
;	non-digit.
;
;	If input is non-string, converts to default-formtat trimmed string and returns. 
;
;	Valid hex digits are 0..9, a..f, A..F. Minus otional by keyword.
;
; USES:
;
; HISTORY:
;	2009 Nov 04  F. Morgan: First version written and tested.
;	2009 Nov 04  FM. Add non-string input return. Add allow_float.
;
;******************************************************************************

function cat_strip_nondigit, input_string, allow_hex=allow_hex, $
          allow_minus=allow_minus, allow_float=allow_float


forward_function cat_is_digit

if (size(input_string,/type) ne 7) then return,strtrim(input_string,2)

digit_flags = cat_is_digit(input_string,allow_hex=allow_hex, $
                allow_minus=allow_minus, allow_float=allow_float)

k = where(digit_flags, nk)
if (nk eq 0) then begin
	return,''
endif else begin
	xseq = k - k[0] - indgen(nk)
	first_digit = k[where((xseq eq 0), nseq)]
	return,strmid(input_string, first_digit[0], nseq)
endelse

return,''   ; Don't expect to get here
end
