;******************************************************************************
;
; function crism_bin_code, pixel_averaging_width
;
; Return the CRISM bin code given number of pixels binned.
;
; INPUT:
;	pixel_averaging_width   string or int     Number of pixels binned (PDS label
;	                                          parameter PIXEL_AVERAGING_WIDTH)
;
; OUTPUT: no parameters
;
; KEYWORDS: None
;
; RETURN VALUE:  bin code (0,1,2,3, or -1 if invalid pixel_averaging_width input)
;
; NOTES:
;
; USES:
;	cat_strip_quotes
;
; HISTORY:
;	2009 Jan 28  F.Morgan  First version. 
;
;******************************************************************************


function crism_bin_code, pixel_averaging_width

forward_function cat_strip_quotes

if (size(pixel_averaging_width, /type) eq 7) then begin
	paw = fix(cat_strip_quotes(pixel_averaging_width))
endif else begin
	paw = round(pixel_averaging_width)   ; in case of float input
endelse

case paw of
	1:  bc=0
	10: bc=3
	2:  bc=1
	5:  bc=2
	else: bc = -1
endcase
return,bc
end
