;***************************************************************************************
;
; function pdsfile_is_spectral, fname
;
; Determine whether a PDS data file is spectral scene data or not 
; (as opposed to DDR, etc). Filename must be PDS derivative, but 
; can be CAT-appended.
;
; INPUT PARAMETERS:
;	fname   	string		PDS filename (possibly derivative, eg [pds]_CAT.IMG)
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:
;	int = 0 (not spectral scene data), or 1 (is spectral)
;
; KEYWORDS  none
;
; EXAMPLES:
;
; HISTORY:
;	2009 Nov 24   FM. First version
;	2009 Nov 27   FM. Documentation edit.
;	2012 Mar 15   FM  update to recognize MTRDR related files
;	2012 Mar 17   FM  Dont call MTR DE file spectral (makes it load 
;                     w/o band names)
;	2015 Oct 08   FM  Bug fix: Dont say TER or MTR summary products are spectral:
;                     changed structure. Independent returns instead of 1 long conditional. 
;                     _TER and _MTR not enough, require _IF also, so _SR*_MTR is not 
;                     called spectral.
;	2016 Jan 08   FM  Bug fix: Add RA TRR to list of spectral files
;
;***************************************************************************************

function pdsfile_is_spectral, fname

if (n_elements(fname[0]) eq 0) then return,0
fnmup = strupcase(strtrim(file_basename(fname[0]),2))
if (fnmup eq '') then return,0

if (strpos(fnmup,'_EDR') gt 0) then return,1
if (strpos(fnmup,'_MRRIF_') gt 0) then return,1
if (strpos(fnmup,'_MRRAL_') gt 0) then return,1
if ((strpos(fnmup,'_TRR') gt 0) and (strpos(fnmup,'_IF') gt 0)) then return,1
if ((strpos(fnmup,'_TRR') gt 0) and (strpos(fnmup,'_AL') gt 0)) then return,1
if ((strpos(fnmup,'_TRR') gt 0) and (strpos(fnmup,'_RA') gt 0)) then return,1
if ((strpos(fnmup,'_MTR') gt 0) and (strpos(fnmup,'_IF') gt 0)) then return,1
if ((strpos(fnmup,'_TER') gt 0) and (strpos(fnmup,'_IF') gt 0)) then return,1



end
