;******************************************************************************
;
; function find_wavelength_file, img_fnm
;
; Given a filename for an image file, find the wavelength file specified 
; in ts label. (No interactive search, and no search for alternate files.)
;
; INPUT:
;	img_fnm     string     Image tile filename 
;
; OUTPUT: no parameters
;
; RETURN VALUE:
;	Wavelength file filename, '' if fail.
;
; KEYWORD:
;	AUX_PATH  Path to CAT aux_files dir; use in DIL to prevent interactive dialog
;
; NOTES:
;
; HISTORY:
;	2009 Sep 16   Frank Morgan: First version
;	2013 Jun 10   FM: Add aux_path and code to use it
;
;******************************************************************************

function find_wavelength_file, img_fnm, aux_path=aux_path

forward_function cat_auxfile_path, reconstruct_pds_path

fnd_file = ''
psep = path_sep()
find_lblfile, img_fnm, img_lbl, /batch
if (file_test(img_lbl)) then begin
	read_crism_lbl, img_lbl[0], ['MRO:WAVELENGTH_FILE_NAME'], wvfnm
	wvfnm = strtrim(cat_strip_quotes(wvfnm[0]),2)
	if (wvfnm ne '') then begin
		; Find the wavelength file. First, look in same directory as input file...
		try_file = file_dirname(img_fnm,/mark_directory) + wvfnm
		fnd_file = (file_search(try_file,/fold_case))[0]
		if (fnd_file eq '') then begin
			; Not there, look in CAT CDR collection
			auxp = ''
			if (size(aux_path,/type) eq 7) then begin
				if (file_test(aux_path[0])) then begin
					auxp = strtrim(aux_path[0],2)
					if (strmid(auxp,strlen(auxp)-1,1) ne path_sep()) then auxp += path_sep()
				endif
			endif
			if (auxp eq '') then auxp=cat_auxfile_path()
			try_file = auxp + 'CDRs' + psep + '*' + psep + wvfnm
			fnd_file = (file_search(try_file,/fold_case))[0]
			if (fnd_file eq '') then begin
				; Not there, look in PDS archive:
				try_file = reconstruct_pds_path(wvfnm) + wvfnm
				fnd_file = (file_search(try_file,/fold_case))[0]
			endif
		endif
	endif
endif
wvfnm = ''
if (file_test(fnd_file)) then wvfnm=fnd_file
return,wvfnm
end
