;******************************************************************************
;
; function get_wc_coeff, sclk
;
; Load WC coefficiens from WC CDR for a given time.
;
; INPUT:
;	sclk       long or string   Observation SCLK  (can be array)
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE
;	fltarr(3) = Wavelength shift coefficients, (fail: 65535.0)
;
; KEYWORDS:
;	aux_path     Path to CAT aux_files; prevents cat_auxfile_path() call in list_all_dr
;
; NOTES:
;
; USES:
;	list_all_dr
;	select_dr
;
; HISTORY:
;	2009 Sep 28 F. Morgan   First version
;
;******************************************************************************

function get_wc_coeff, sclk, aux_path=aux_path

forward_function list_all_dr, select_dr

; List all the WC CDRs:
allwc = list_all_dr(6, 'WC', aux_path=aux_path)

nobs = n_elements(sclk)
wccoeff = fltarr(3,nobs)
x = fltarr(3)
for i=0,nobs-1 do begin
	this_wc = select_dr(allwc, 6, 'WC', sclk[i], 'L', 0, 0)
	if (this_wc eq '') then begin
		wccoeff[*,i] = 65535.
	endif else begin
		openr,uwc, this_wc, /get_lun
		readf,uwc, x
		free_lun,uwc
		wccoeff[*,i] = x
	endelse
endfor
return,wccoeff
end
