;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;+
; $Id: pick1_from_list.pro,v 1.00 2008/08/15 13:52:05 swartz Exp $
;
; NAME:
;       PICK1_FROM_LIST
;
; PURPOSE:
;       This function allows interactive selection of one string 
;       from a list of strings supplied in a string array.
;
; CATEGORY:
;       Compound widget.
;
; CALLING SEQUENCE:
;       function pick1_from_list, xlist, wndo_title
;
; INPUTS:
;       XLIST:        String array containing choices
;       WNDO_TITLE:   Scalar string for titling the interactive selection window
;
; KEYWORD PARAMETERS:
;       user_cancelled   Named variable, set to 1 on output if user cancelled; 
;                        otherwise 0.
;
; OUTPUTS PARAMETER:
;       None.
; RETURN VALUE:
;       String = Selected string item, or "" if cancelled or failed.
;
; MODIFICATION HISTORY:
;       Written by: F. Morgan, SRA, 8/13/08
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;***************************************************************************
;***************************************************************************
;; Function to interrogate widget values, before passing values to writer. *
;***************************************************************************
;***************************************************************************
function query_pfl_widgets, state

;;; Interrogate the widgets for values

for i=0, state.nitems-1 do begin
	if (widget_info(state.id.lsbutton[i], /button_set)) then break
endfor
if (i lt state.nitems) then begin
	state.input.lstslct = i
endif else begin
	state.input.lstslct = -1
endelse

;i = 0
;while ((i lt state.nitems) && (~widget_info(state.id.lsbutton[i], /button_set))) do i++
;if ((i lt state.nitems) && (widget_info(state.id.lsbutton[i], /button_set))) then begin
;	state.input.lstslct = i
;endif else begin
;	state.input.lstslct = -1
;endelse

return, state
end



;***************************************************************************
;***************************************************************************
;; Event handler.  All widget event are processed here.                   **
;***************************************************************************
;***************************************************************************
function pick1_from_list_event, ev

common pickrslt, idxslct

;message, 'Event.', /continue
cache = widget_info(ev.handler, /child)
widget_control, cache, get_uvalue = state

;;; Processing events
case ev.id of
;;Main Menu*****************************************************************

    state.id.ok: begin
        ; User clicked OK, must have selected their file:
        state = query_pfl_widgets(state)
        idxslct = state.input.lstslct
        widget_control, state.id.main, /destroy
    end
    
    state.id.cancel: begin
        ; print,' USER CANCELED.'
        idxslct = -1
        widget_control, state.id.main, /destroy
    end

    else: begin
        ;; Did not identify the event or it's a droplist
;        message, 'Could not indentify event.', /continue
;        help, ev, /structure
    end

endcase



return, 0
end


;***************************************************************************
;***************************************************************************
;; Event handler wrapper.                                                 **
;***************************************************************************
;***************************************************************************
pro pick1_from_list_eventp, ev
;; This event procedure is used when the compound widget operates as a
;; standalone widget (since for XMANAGER the event handler must be a
;; procedure and it can not return a value).  We just use it as a
;; wrapper for our event function above.
dummy = pick1_from_list_event(ev)
return
end


;***************************************************************************
;***************************************************************************
;; Main routine that creates compound widget.                             **
;***************************************************************************
;***************************************************************************
function pick1_from_list, xlist, wndo_title, user_cancelled=user_cancelled

common pickrslt, idxslct


user_cancelled = 0

;;; The STATE structure
nitems = n_elements(xlist)
id = { main:0L, ok:0L, cancel:0L, lsbutton: lonarr(nitems) }
input = { lstslct: 0L }
state = { id: id, $
          input: input, $
          nitems: nitems $
        }


;;; Main base
if (n_elements(wndo_title) eq 0) then wndo_title='Select From List'
if (size(wndo_title, /type) ne 7) then wndo_title='Select From List'
main = widget_base(title=wndo_title)
state.id.main = main

;; Definitions of the compund widget sub-components.
;; Since this is the first child widget of the main base the state
;; will be stored in the uvalue of this widget.
inner = widget_base(main, column = 1)
                           
;;Main Menu*****************************************************************

; Item list... useful hint from http://www.dfanning.com/widget_tips/checkbox.html
longest_str = strlen(wndo_title) > max(strlen(xlist))
xs = (longest_str * 8L < 500) > 250
yscl = 25L
lsbase = widget_base(inner, /exclusive, /scroll, y_scroll_size=(nitems*yscl < 12*yscl), x_scroll_size=xs)
for i=0,n_elements(xlist)-1 do state.id.lsbutton[i] = widget_button(lsbase,value=xlist[i])
widget_control, state.id.lsbutton[0], Set_Button=1 

; OK/cancel...
okbar = widget_base(inner, row=1)
state.id.ok = widget_button(okbar, value='OK', xsize=0.8, units=1)        ; width ~0.8inches
state.id.cancel = widget_button(okbar, value='Cancel', xsize=0.8, units=1)


;; Done creating widgets.

;; Save STATE in the USER value of the first child widget.
;; NO_COPY speeds this statement but makes STATE undefined afterwards.
widget_control, widget_info(main, /child), set_uvalue=state, /no_copy

widget_control, main, /realize
xmanager, 'pick1_from_list', main, event_handler = 'pick1_from_list_eventp'


;print, 'OUT: lstslct = ', idxslct
if (n_elements(idxslct) eq 0) then return,""
if (idxslct ge n_elements(xlist)) then return,""
if (idxslct lt 0) then begin
	user_cancelled = 1
	return,""
endif
return, xlist[idxslct]

end
