;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Event handler for processing option selection:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; 2009-Mar-10  FM  Commented out thermal correction - marked by "; thermal edit"
; 2009-Mar-11  FM  Changed order of buttons - phot, therm, atm
; 2009-Oct-28  FM  Added choice whether to allow spectral subsetting

pro pops_widget_event, ev

; Recover the state:
widget_control,ev.top,get_uvalue=state

; See which button was pressed:
;  only do things if OK or CANCEL.
widget_control,ev.id,get_uvalue=uvalue

; If OK, get settings:
if (uvalue eq 'OK') then begin

  widget_control,state.atmgrp,get_value=atmindex
  if (atmindex eq 0) then atm='scale'
  if (atmindex eq 1) then atm='epf'
  if (atmindex eq 2) then atm='none'

  ;widget_control,state.thermgrp,get_value=thermindex  ; thermal edit
  ;if (thermindex eq 0) then therm='algm'
  ;if (thermindex eq 1) then therm='algk'
  ;if (thermindex eq 2) then therm='none'
  therm = 'none'                                ; thermal edit DELETE THIS IF RESTORED

  widget_control,state.photgrp,get_value=photindex
  if (photindex eq 0) then phot='on'
  if (photindex eq 1) then phot='off'

  subalw = 'no'  ; default
  k = where((strupcase(strtrim(tag_names(state),2)) eq 'SUBSETGRP'),nk)
  if (nk eq 1) then begin
    widget_control,state.subsetgrp,get_value=suballow
    if (suballow eq 0) then subalw='no'
    if (suballow eq 1) then subalw='yes'
  endif
  
  ; Send info to pointer location that lives outside of widget
  (*state.ptr).choices=[atm,therm,phot,subalw]

  ;Kill the GUI
  widget_control,ev.top,/destroy
endif

; If CANCEL, kill GUI:
if (uvalue eq 'CANCEL') then begin
  (*state.ptr).choices=['cancel','cancel','cancel','cancel']
  widget_control,ev.top,/destroy
endif

end


;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Program defining the GUI:
;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro pops_widget, choices

; Devel version?
devver = strtrim(get_catcfg_param('CAT_DEV_VERSION'),2)
if (devver ne '1') then dv=0 else dv=1

; Create a top level base that holds everything:
pops=widget_base(TITLE='ATP Corrections',/column)

; Define options for buttons:
atmops=['Division by scaled volcano observation.','Empirical use of EPF.','None.']
photops=['On','Off']
thermops=['Empirical temp estimate from values at 2.3 mircron (algM).',$
          'Modeled temp estimate from ADR-TE (algK).','None.']
subops = ['No', 'Yes']

; Create a comound widget for the choices; set default values to no correction:
photgrp=cw_bgroup(pops,photops,uvalue='photchoice',set_value=1,$
                 label_top='Photometric correction: Division by cos(i):',$
                 /column,/exclusive,/frame)

;thermgrp=cw_bgroup(pops,thermops,uvalue='thermchoice',set_value=2,$   ; thermal edit
;          label_top='Choose thermal correction:                                    ',$
;                 /column,/exclusive,/frame)

atmgrp=cw_bgroup(pops,atmops,uvalue='atmchoice',set_value=2,$
                 label_top='Choose atmospheric correction:             ',$
                 /column,/exclusive,/frame)

if (dv) then subsetgrp = cw_bgroup(pops,subops,uvalue='subsetchoice',set_value=0,$
                 label_top='Allow spectral subsetting?          ',$
                 /column,/exclusive,/frame)


; Create an empty space for asthetics:
spacer=widget_base(pops,/row)
gap=widget_label(spacer,value='')

; Create exit buttons:
activate = widget_base(pops,/row)
sel1 = widget_button(activate,value='OK',uvalue='OK',xsize=50,ysize=30)
sel2 = widget_button(activate,value='CANCEL',uvalue='CANCEL',xsize=50,ysize=30)

; Create a pointer location where choices can be stored "off-site" from
;   widget:
ptr = Ptr_New({choices:['','','','']})

; Keep useful information in a state structure:
;state =  {atmgrp:atmgrp, thermgrp:thermgrp, photgrp:photgrp, ptr:ptr}    ; thermal edit
if (dv) then begin
  state =  {atmgrp:atmgrp, photgrp:photgrp, subsetgrp:subsetgrp, ptr:ptr}
endif else begin
  state =  {atmgrp:atmgrp, photgrp:photgrp, ptr:ptr}
endelse

; Save the state structure in the base uvalue and realize application:
widget_control, pops, set_uvalue=state, /realize

;Handoff control of application to xmanager:
xmanager,'pops_widget',pops

; Get info from "off-site" location and clear pointer
choices = (*ptr).choices
ptr_free,ptr

end
