; select_scaling_art.pro
;
; Widget to select volcano scan scaling wavelengths and to 
; choose whether to apply artifact correction. 
;
; Returns intarr(2):
;               /  0, New McGuire 2-wavelength set
;    arr[0] =  <   1, old Pelkey 2-wavelength set
;               \  2, PCM multi-wavelength set
;
;    arr[1] = 0, disable artifact correction; 1, enable
;    arr[*] = -1: User cancelled
;
;*********************************************************************
;*********************************************************************


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Event handler for processing option selection:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro scl_widget_event, ev

; Recover the state:
widget_control, ev.top, get_uvalue=state

; See which button was pressed:
;  only do things if OK or CANCEL.
widget_control, ev.id, get_uvalue=uvalue

; If OK, get settings:
if (uvalue eq 'OK') then begin

  widget_control, state.bandgrp, get_value=bandindex

  widget_control, state.artgrp, get_value=artval
  enableart = artval[0]
  
  ; Send info to pointer location that lives outside of widget
  (*state.ptr).choices=[bandindex, enableart]

  ;Kill the GUI
  widget_control,ev.top,/destroy
endif

; If CANCEL, kill GUI:
if (uvalue eq 'CANCEL') then begin
  (*state.ptr).choices=[-1,-1]
  widget_control,ev.top,/destroy
endif

end



;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Program defining the GUI:
;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro select_scaling_art, choices


; Create a top level base that holds everything:
scl = widget_base(TITLE='Scaling and Artifact Correction',/column)


; List of scaling band set options:
devver = strtrim(get_catcfg_param('CAT_DEV_VERSION'),2)
if (devver ne '1') then dv=0 else dv=1
if (dv) then begin
	; Development version, include the McGuire multi-wavelength option
	bandset_ops = [ 'New McGuire 2-wavelength (2007/1980)', $
                'Old Pelkey 2-wavelength (2011/1899)', $
                'McGuire multi-wavelength (slow, little difference)' ]
endif else begin
	; Release version, don't include it:
	bandset_ops = [ 'New McGuire 2-wavelength (2007/1980)', $
                'Old Pelkey 2-wavelength (2011/1899)' ]
endelse

; Create a comound widget for the choices
bandgrp = cw_bgroup(scl,bandset_ops,uvalue='bandschoice',set_value=0, $
                 label_top='Choose bands for volcano scan scaling:', $
                 /column,/exclusive,/frame)

artgrp = cw_bgroup(scl,['Enable artifact correction?'], uvalue='artchoice', set_value=[1], $
                 /column,/nonexclusive,/frame)


; Create an empty space for asthetics:
spacer=widget_base(scl,/row)
gap=widget_label(spacer,value='')

; Create exit buttons:
activate = widget_base(scl,/row)
sel1 = widget_button(activate,value='OK',uvalue='OK',xsize=50,ysize=30)
sel2 = widget_button(activate,value='CANCEL',uvalue='CANCEL',xsize=50,ysize=30)

; Create a pointer location where choices can be stored "off-site" from
;   widget:
ptr = Ptr_New({choices:[-1,-1]})

; Keep useful information in a state structure:
state =  {bandgrp:bandgrp, artgrp:artgrp,  ptr:ptr}

; Save the state structure in the base uvalue and realize application:
widget_control, scl, set_uvalue=state, /realize

;Handoff control of application to xmanager:
xmanager,'scl_widget',scl

; Get info from "off-site" location and clear pointer
choices = (*ptr).choices
ptr_free,ptr

end
