; select_summary_params.pro
;
; Widget to select which summary parameters to compute 
;
; Returns selected indices in parameter selected_params
;	selected_params = intarr of indices
;	selected_params = -1 if users cancels
;
; Also - user decides whether to compute in memory or to file, and
; selects output filename if needed.
;
;*********************************************************************
;*********************************************************************


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Event handler for processing option selection:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro sumpar_widget_event, ev

; Recover the state:
widget_control, ev.top, get_uvalue=state

; See which button was pressed:
;  only do things if OK or CANCEL.
widget_control, ev.id, get_uvalue=uvalue

; If OK, get settings:
if (uvalue eq 'OK') then begin

	widget_control, state.basicgroup, get_value=basic_index
	widget_control, state.advgroup, get_value=adv_index

  
	; Send info to pointer location that lives outside of widget
	if (state.nbasic gt 0) then (*(state.ptr_basic)) = basic_index else (*(state.ptr_basic)) = [0]
	if (state.nadv gt 0) then (*(state.ptr_adv)) = adv_index else (*(state.ptr_adv)) = [0]

	;Kill the GUI
	widget_control,ev.top,/destroy
endif

; If CANCEL, kill GUI:
if (uvalue eq 'CANCEL') then begin
	(*state.ptr_basic) = [0]
	(*state.ptr_adv) = [0]
	widget_control,ev.top,/destroy
endif

; If set selection, sensitize or desensitize the advanced list:
if (uvalue eq 'setchoice') then begin
	widget_control, state.setgrp, get_value=setsel
	if (setsel eq 0) then begin
;		widget_control, state.advgroup, set_value=replicate(0,state.nadv)
;		widget_control, state.advgroup, sensitive=0
		widget_control, state.advgroup, set_value=replicate(0,state.nadv)
		widget_control, state.adv_base, sensitive=0
	endif else begin
;		widget_control, state.advgroup, sensitive=1
;		widget_control, state.advgroup, set_value=replicate(1,state.nadv)
		widget_control, state.adv_base, sensitive=1
		widget_control, state.advgroup, set_value=replicate(1,state.nadv)
	endelse
endif

if (uvalue eq 'basicall') then begin
	; Select or deselect all basic parameters
	widget_control, state.basic_all, get_value=baval
	if (baval eq 'Deselect all') then begin
		; Deselect all...
		widget_control, state.basicgroup, get_value=basic_index
		if (n_elements(basic_index) gt 0) then begin
			basic_index[*] = 0
			widget_control, state.basicgroup, set_value=basic_index
		endif
		; ...and toggle button to select all:
		widget_control, state.basic_all, set_value='Select all'
	endif else if (baval eq 'Select all') then begin
		; Select all...
		widget_control, state.basicgroup, get_value=basic_index
		if (n_elements(basic_index) gt 0) then begin
			basic_index[*] = 1
			widget_control, state.basicgroup, set_value=basic_index
		endif
		; ...and toggle button to deselect all:
		widget_control, state.basic_all, set_value='Deselect all'
	endif
endif

if (uvalue eq 'advall') then begin
	; Select or deselect all advanced parameters
	widget_control, state.adv_all, get_value=aaval
	if (aaval eq 'Deselect all') then begin
		; Deselect all...
		widget_control, state.advgroup, get_value=adv_index
		if (n_elements(adv_index) gt 0) then begin
			adv_index[*] = 0
			widget_control, state.advgroup, set_value=adv_index
		endif
		; ...and toggle button to select all:
		widget_control, state.adv_all, set_value='Select all'
	endif else if (aaval eq 'Select all') then begin
		; Select all...
		widget_control, state.advgroup, get_value=adv_index
		if (n_elements(adv_index) gt 0) then begin
			adv_index[*] = 1
			widget_control, state.advgroup, set_value=adv_index
		endif
		; ...and toggle button to deselect all:
		widget_control, state.adv_all, set_value='Deselect all'
	endif
endif

return
end



;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Program defining the GUI:
;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro select_summary_params, det, summary_list, adv_basic, selected_params, out_fnm

; INPUT:
;	det           int        CRISM detector: 0=VNIR, 1=IR, 2=Joined
;	summary_list  strarr(n)  List of all optional parameter names
;	adv_basic     intarr(n)  Int array matching options; 1=basic, 2=advanced
;	out_fnm       string     Default output filename (optional)
;
; OUTPUT:
;	selected_params  intarr(nsel) nsel <= n; Indices into summary_list of selected parameters 
;	out_fnm       string     Output filename; to memory if ''


selected_params = -1    ; indicates cancel

case det of
	0: detstr = 'VNIR'
	1: detstr = 'IR'
	2: detstr = 'Joined'
	else: detstr = ''
endcase


; Default selections:
nparam = n_elements(summary_list)
setidx = intarr(nparam)
devver = strtrim(get_catcfg_param('CAT_DEV_VERSION'),2)
if (devver eq '1') then begin
	; Set default = all params
	default_set = 1
	setidx[*] = 1
endif else begin
	; default = limited set
	default_set = 0
	k = where((adv_basic eq 1), nk)
	if (nk gt 0) then setidx[k]=1	
endelse


; Separate into basic and advanced parameter lists
basic_idx = where((adv_basic eq 1), nbasic)
if (nbasic gt 0) then basic_params=summary_list[basic_idx] else basic_params=''
adv_idx = where((adv_basic eq 2), nadv)
if (nadv gt 0) then adv_params=summary_list[adv_idx] else adv_params=''



; Create a top level base that holds everything:
wbase = widget_base(TITLE=detstr+' Parameter Selection',/column)

; Create widgets for the choices
maxy = long(0.25*(get_screen_size())[1])
miny = 30
yscl = 22L    ; pixels per parameter, approx
setbasic = replicate(1,nbasic)
bysz = ((nbasic*yscl) < maxy) > miny
basic_base = widget_base(wbase, /row, /base_align_center, uvalue='basicbase')
  basicgroup = cw_bgroup(basic_base,basic_params,uvalue='basicchoice',set_value=setbasic, $
                 label_top='Choose '+detstr+' summary parameters:', $
                 /column,/nonexclusive,/frame, y_scroll_size=bysz, /scroll)
  basic_all = widget_button(basic_base, value='Deselect all',uvalue='basicall')
setadv = replicate(default_set,nadv)
aysz = ((nadv*yscl) < maxy) > miny
adv_base = widget_base(wbase, /row, /base_align_center, uvalue='advbase')
  advgroup = cw_bgroup(adv_base,adv_params,uvalue='advchoice',set_value=setadv, $
                 /column,/nonexclusive,/frame, y_scroll_size=aysz, /scroll)
  adv_all = widget_button(adv_base, value='Deselect all',uvalue='advall')


; Buttons to select all params or basic set:
list_select = ['Basic parameters', 'All parameters']
setgrp = cw_bgroup(wbase,list_select, uvalue='setchoice', set_value=default_set, $
                 /column,/exclusive,/frame )


; Create an empty space for asthetics:
spacer = widget_base(wbase,/row)
gap = widget_label(spacer,value='')

; Create exit buttons:
activate = widget_base(wbase,/row)
sel1 = widget_button(activate,value='OK',uvalue='OK',xsize=50,ysize=30)
sel2 = widget_button(activate,value='CANCEL',uvalue='CANCEL',xsize=50,ysize=30)

; Create a pointer location where choices can be stored "off-site" from
;   widget:
if (nbasic gt 0) then ptr_basic=ptr_new(intarr(nbasic)) else ptr_basic=0
if (nadv gt 0) then ptr_adv=ptr_new(intarr(nadv)) else ptr_adv=0

; Keep useful information in a state structure:
state =  {basic_base: basic_base, $
          basicgroup: basicgroup, $
          basic_all: basic_all, $
          advgroup: advgroup, $
          adv_all: adv_all, $
          adv_base: adv_base, $
          setgrp: setgrp, $
          ptr_basic: ptr_basic, $
          nadv: nadv, $
          nbasic: nbasic, $
          ptr_adv: ptr_adv }

; Save the state structure in the base uvalue and realize application:
widget_control, wbase, set_uvalue=state, /realize

; If default set is basic, desensitize the advanced parameters:
;if (default_set eq 0) then widget_control, advgroup, sensitive=0
if (default_set eq 0) then widget_control, adv_base, sensitive=0


;Handoff control of application to xmanager:
xmanager,'sumpar_widget',wbase


; Get info from "off-site" location and clear pointers
noparam = 1   ; will flag if no parameters selected
if (nbasic gt 0) then begin
	k = where((*state.ptr_basic ne 0), nbasicsel)
	if (nbasicsel gt 0) then begin
		basic_in = basic_idx[k]
		noparam = 0
	endif
endif
if (nadv gt 0) then begin
	k = where((*state.ptr_adv ne 0), nadvsel)
	if (nadvsel gt 0) then begin
		adv_in = adv_idx[k]
		noparam = 0
	endif
endif
if (noparam) then begin
	; No parameters at all selected:
	selected_params = -1
	return
endif

; Combine advanced and basic parameters
if (nbasicsel eq 0) then begin
	selected_params = adv_in
endif else if (nadvsel eq 0) then begin
	selected_params = basic_in
endif else begin
	selected_params = [basic_in, adv_in]
endelse

ptr_free,ptr_basic
ptr_free,ptr_adv

return
end
