;******************************************************************************
;
; function strip_catfile_appendage, fnm
;
; Strip any CAT-appended text in a filename between the PDS-legit filename 
; and the extension.
;
; INPUT:
;	fnm       string     PDS filename, possibly with CAT-appended text
;	                     preceeding extension
;
; OUTPUT: No parameters
;
; RETURN VALUE
;	Filename stripped of CAT baggage.  If input does not match any expected 
;	PDS root, then returns filename unchanged.
;
; KEYWORDS: None.
;
; NOTES:
;	These are the possible legitimate filenames:
;		cdr4: CDR410803692813_WA0200030L_3.*
;		cdr6: CDR6_1_0000000000_SW_L_3.LBL
;		trdr: FRT0000B8F7_07_IF168L_TRR2.IMG
;		ddr:  FRT00005910_07_DE163S_DDR1.IMG
;		tile: T1260_MRRIF_20N128_0256_1.IMG
;
; USES:
;
; HISTORY:
;
;******************************************************************************

function strip_catfile_appendage, fnm


trfnm = strtrim(fnm,2)
fpath = file_dirname(trfnm)
fnmbase = file_basename(trfnm)
fnmbase_up = strupcase(fnmbase)
p = strpos(fnm,'.',/reverse_search)
fext = strmid(fnm,p)     ; includes .

if ((fpath eq '.') and (strmid(trfnm,0,1) ne '.')) then begin
	fpath = ''
endif else begin
	if (strmid(fpath,strlen(fpath)-1,1) ne path_sep()) then fpath += path_sep()
endelse

segs = strsplit(fnmbase,'_',/extract)
if (strlen(segs[0]) eq 11) then begin
	; Input file is TRDR, expect form like FRT0000B8F7_07_IF168L_TRR2.IMG
	newbase = segs[0] + '_' + segs[1] + '_' + segs[2] + '_' + strmid(segs[3],0,4)
endif else if (strmid(fnmbase_up,0,4) eq 'CDR4') then begin
	; Input file is CDR4, expect form like CDR410803692813_WA0200030L_3.*
	newbase = segs[0] + '_' + segs[1] + '_' + strmid(segs[2],0,1)
endif else if (strmid(fnmbase_up,0,4) eq 'CDR6') then begin
	; Input file is CDR6, expect form like CDR6_1_0000000000_SW_L_3.LBL
	newbase = segs[0] + '_' + segs[1] + '_' + segs[2] + '_' + segs[3] + '_' $
	            + segs[4] + '_' + strmid(segs[5],0,1)
endif else if ((strmid(fnmbase_up,0,1) eq 'T') and (strlen(segs[0]) eq 5)) then begin
	; Input file is tile, expect form like T1260_MRRIF_20N128_0256_1.IMG
	newbase = segs[0] + '_' + segs[1] + '_' + segs[2] + '_' + segs[3] + '_' + strmid(segs[4],0,1)
endif else begin
	; Input not recognized as PDS-derived filename
	return, fnm
endelse

; Reattach directory and extension; if first character of input basename uper case,
; assume extension should be upper case too
x1b = (byte(strmid(fnmbase,0,1)))[0]
if ((x1b ge (byte('A'))[0]) and (x1b le (byte('Z'))[0])) then begin
	fext = strupcase(fext)
endif else begin
	fext = strlowcase(fext)
endelse
stripfile = fpath + newbase + fext

return, stripfile
end
