function check4file_exist, fname

; Utility function for byte_order_family(); returns 1 if input filename 
; exists, 0 if not. Handles IDL versions too early for file_test().

if (float(!version.release) ge 5.4) then begin
	return, file_test(fname)
endif else begin
	if (strpos(strupcase(!version.os_family),'UNIX') ge 0) then begin
		spawn,'ls '+fname, res
	endif else begin
		spawn,'dir '+fname, res
	endelse
	if (strlen(strtrim(res[0])) gt 0) then return,1 else return,0
endelse
end



pro byte_order_family

; This routine checks whether you machine belongs in the 
; x86_family or sparc_family, for purposes of the CRISM 
; thermal model IDL code.
;
; It figures this out by opening a file in the current directory, 
; writing an integer (2-byte) value = 1 to it, then reading it back in 
; as a bytarr(2) and checking which byte is 1. 
;
; It'll clean up the file when done, and checks first to make sure it 
; doesn't already exist before it creates it to begin with.
;
; OLD IDL VERSIONS:
;	This routine will not compile on IDL earlier than version 5.4 because 
;	that's when file_test() was introduced. 
;	If needed, you can comment out file_test() and just make sure to 
;	run  this code in a directory where you don't already have a file 
;	called tmp_file_for_byteord_chk.dat.



; First generate a file name that doesn't exist in the current directory
fname = 'tmp_file_for_byteord_chk.dat'
num = 0
while ( (check4file_exist(fname)) AND (num lt 100) ) do begin
	fname = 'tmp_file_for_byteord_chk' + strtrim(num,2) + '.dat'
	num = num+1
endwhile
if (check4file_exist(fname)) then begin
	print,' BYTE_ORDER_FAMILY.PRO wanted to write to a file like tmp_file_for_byteord_chkXXX.dat'
	print,' but could not find a name that did not already exist. Giving up.'
	return
endif

on_ioerror, BADIO

x_int = 1
openw,u, fname, /get_lun
writeu, u, x_int
close,u

; Now read it back:
b = bytarr(2)
openr,u, fname
readu, u, b
free_lun,u

print,''
print, " *****  Your architecture is '" + !version.arch + "'  *****"
if (b[0] eq 1) then begin
	print, ' *****  That belongs in the x86_family.  *****'
endif else begin
	print, ' *****  That belongs in the sparc_family.  *****'
endelse

print, ''
print, ' note: if you add your architecture to the types_instruc structure, '
print, "        be sure to include all the spaces between the quotes if any."

; Clean up the file:
if (float(!version.release) ge 5.4) then begin
	file_delete, fname
endif else begin
	print,' Please delete file '+fname
endelse
return

BADIO: print, 'BYTE_ORDER_FAMILY.PRO  failed on an I/O problem.'
print, ' You may need to clean up file '+fname
return
end
